#!/usr/bin/env python 
# encoding: utf-8 

 
## Module infomation ### 
# Python (2.7.14) 
# numpy (1.13.3+mkl) 
# PyAudio (0.2.11) 
# pyqtgraph (0.10.0) 
# PyQt4 (4.11.4) 
# All 32bit edition 
######################## 
 
import wave

import pyaudio 
import numpy as np 
import pyqtgraph as pg 
from pyqtgraph.Qt import QtGui, QtCore 

wavfile ="test48k.wav"
     
class SpectrumAnalyzer: 
     CHANNELS = 1 
     RATE = 48000 
     CHUNK = 1024
     START = 0 
     N = 1024
     WAVE_RANGE = 1 
     SPECTRUM_RANGE = -60 
     UPDATE_SECOND = 10
     
     def __init__(self):

         self.wr = wave.open(wavfile, "rb")
         
         print "Channel num : ", self.wr.getnchannels()
         print "Sample size : ", self.wr.getsampwidth()
         print "Sampling rate : ", self.wr.getframerate()
         print "Frame num : ", self.wr.getnframes()
         print "Parms : ", self.wr.getparams()
         print "Sec : ", float(self.wr.getnframes()) / self.wr.getframerate()

         #Pyaudio Configuration
         self.pa = pyaudio.PyAudio() 
          
         self.stream = self.pa.open(format = self.pa.get_format_from_width(self.wr.getsampwidth()),
                  channels = self.wr.getnchannels(),
                  rate = self.wr.getframerate(),
                  output = True)
         
         # Graph configuration 
         # Application 
         self.app = QtGui.QApplication([]) 
         self.app.quitOnLastWindowClosed() 
         # Window 
         self.win = QtGui.QMainWindow() 
         self.win.setWindowTitle("SpectrumAnalyzer") 
         self.win.resize(800, 600) 
         self.centralwid = QtGui.QWidget() 
         self.win.setCentralWidget(self.centralwid)  
         # Layout 
         self.lay = QtGui.QVBoxLayout() 
         self.centralwid.setLayout(self.lay) 
         # Wave figure window setting 
         self.plotwid1 = pg.PlotWidget(name="wave") 
         self.plotitem1 = self.plotwid1.getPlotItem() 
         self.plotitem1.setMouseEnabled(x = False, y = False)  
         self.plotitem1.setYRange(self.WAVE_RANGE * -1, self.WAVE_RANGE * 1) 
         self.plotitem1.setXRange(self.START, self.START + self.N, padding = 0) 
         # Spectrum windows setting 
         self.plotwid2 = pg.PlotWidget(name="spectrum") 
         self.plotitem2 = self.plotwid2.getPlotItem()
         self.plotitem2.setLogMode(x = True, y = False)
         self.plotitem2.setMouseEnabled(x = False, y = False)
         #self.plotitem2.setYRange(0, self.SPECTRUM_RANGE)
         self.plotitem2.setYRange(self.SPECTRUM_RANGE, 0)
         #self.plotitem2.setXRange(0, self.RATE / 2, padding = 0)
         # Wave figure Axis 
         self.specAxis1 = self.plotitem1.getAxis("bottom") 
         self.specAxis1.setLabel("Time [sample]") 
         # Spectrum Axis 
         self.specAxis2 = self.plotitem2.getAxis("bottom") 
         self.specAxis2.setLabel("Frequency [Hz]") 
         #Plot data 
         self.curve_wave = self.plotitem1.plot() 
         self.curve_spectrum = self.plotitem2.plot()
         #Widget 
         self.lay.addWidget(self.plotwid1) 
         self.lay.addWidget(self.plotwid2) 
         #Show plot window 
         self.win.show() 
         #Update timer setting 
         self.timer = QtCore.QTimer() 
         self.timer.timeout.connect(self.update) 
         self.timer.start(self.UPDATE_SECOND) 

         self.remain = self.wr.getnframes()
         self.play =1
 

     def update(self): 
         #Get audio input   
         if self.remain > 0:
             data = self.wr.readframes( min (self.CHUNK, self.remain) )
             self.stream.write(data)
             if self.remain > self.CHUNK:
                 data = np.frombuffer(data, dtype= "int16") / 32768.0
                 # Wave figure 
                 wave_figure = data[self.START:self.START + self.N] 
                 # Wave time 
                 wave_time = range(self.START, self.START + self.N) 
                 # Frequency 
                 freqlist = np.fft.fftfreq(self.N, d = 1.0 / self.RATE)  
                 # Spectrum power 
                 x = np.fft.fft(data[self.START:self.START + self.N]) / self.CHUNK 
                 #amplitudeSpectrum = [np.sqrt(c.real ** 2 + c.imag ** 2) for c in x] 
                 amplitudeSpectrum = abs(x) + 1e-6
                 amplitudeSpectrum = 20 * np.log10(amplitudeSpectrum)
                 # Plot setdata 
                 self.curve_wave.setData(wave_time, wave_figure) 
                 self.curve_spectrum.setData(freqlist, amplitudeSpectrum) 
                 self.remain -= self.CHUNK
             else:
                 self.remain -= self.CHUNK
         else:
             if self.play == 1:
                 self.stream.close()
                 self.pa.terminate()
                 self.wr.close()
                 self.play = 0
 
 
if __name__ == "__main__": 
     spec = SpectrumAnalyzer() 
     QtGui.QApplication.instance().exec_() 
